# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Cosmo(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field ccsds_version: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.ccsds_version
    :field packet_type: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.packet_type
    :field secondary_header_flag: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.secondary_header_flag
    :field is_stored_data: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.is_stored_data
    :field application_process_id: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.application_process_id
    :field grouping_flag: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.grouping_flag
    :field sequence_count: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.sequence_count
    :field packet_length: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.packet_length
    :field time_stamp_seconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.time_stamp_seconds
    :field sub_seconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.sub_seconds
    :field padding: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.padding
    :field sc_fsw_28_ver: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_ver
    :field sc_fsw_28_type: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_type
    :field sc_fsw_28_sec_hdr_flag: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_sec_hdr_flag
    :field sc_fsw_28_pkt_apid: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_pkt_apid
    :field sc_fsw_28_seq_flgs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_seq_flgs
    :field sc_fsw_28_seq_ctr: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_seq_ctr
    :field sc_fsw_28_pkt_len: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_pkt_len
    :field sc_fsw_28_shcoarse: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_shcoarse
    :field sc_fsw_28_shfine: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sc_fsw_28_shfine
    :field bcn_adcs_tai_sec: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_tai_sec
    :field bcn_time_since_boot: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_time_since_boot
    :field bcn_hr_cyc_ct_safe: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_hr_cyc_ct_safe
    :field bcn_sec_in_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sec_in_mode
    :field bcn_adcs_bod_rt_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_bod_rt_1
    :field bcn_adcs_bod_rt_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_bod_rt_2
    :field bcn_adcs_bod_rt_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_bod_rt_3
    :field bcn_adcs_att_resid_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_att_resid_1
    :field bcn_adcs_att_resid_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_att_resid_2
    :field bcn_adcs_att_resid_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_att_resid_3
    :field bcn_adcs_gps_pos_ecef_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_gps_pos_ecef_1
    :field bcn_adcs_gps_pos_ecef_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_gps_pos_ecef_2
    :field bcn_adcs_gps_pos_ecef_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_gps_pos_ecef_3
    :field bcn_cmd_rx_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_cmd_rx_ct
    :field bcn_cmd_rej_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_cmd_rej_ct
    :field bcn_cmd_succ_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_cmd_succ_ct
    :field bcn_pld_pass_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pass_ct
    :field bcn_pld_pass_err: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pass_err
    :field bcn_pld_pkt_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pkt_ct
    :field bcn_pld_5_v_0_reg_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_5_v_0_reg_t
    :field bcn_pld_3_v_3_reg_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_3_v_3_reg_t
    :field bcn_pld_sb_1_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_sb_1_t
    :field bcn_pld_sb_2_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_sb_2_t
    :field bcn_pld_vrum_t_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_vrum_t_1
    :field bcn_pld_vrum_t_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_vrum_t_2
    :field bcn_pld_pkt_rx_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pkt_rx_ct
    :field bcn_pld_pkt_ack_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pkt_ack_ct
    :field bcn_pld_pkt_nack_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pkt_nack_ct
    :field bcn_pld_err_reg_0_0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_err_reg_0_0
    :field bcn_pld_err_reg_0_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_err_reg_0_1
    :field bcn_pld_err_reg_1_0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_err_reg_1_0
    :field bcn_pld_err_reg_1_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_err_reg_1_1
    :field bcn_seq_exec_buf_auto: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_exec_buf_auto
    :field bcn_seq_exec_buf_op_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_exec_buf_op_1
    :field bcn_seq_exec_buf_op_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_exec_buf_op_2
    :field bcn_seq_exec_buf_op_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_exec_buf_op_3
    :field reusable_spare_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_1
    :field bcn_store_part_wr_log: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_wr_log
    :field bcn_store_part_rd_log: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_rd_log
    :field bcn_store_part_wr_adcs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_wr_adcs
    :field bcn_store_part_rd_adcs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_rd_adcs
    :field bcn_store_part_wr_hk: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_wr_hk
    :field bcn_store_part_rd_hk: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_rd_hk
    :field bcn_store_part_wr_sci: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_wr_sci
    :field bcn_store_part_rd_sci: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_store_part_rd_sci
    :field bcn_fp_resp_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_resp_ct
    :field bcn_bat_1_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_bat_1_v
    :field bcn_bat_2_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_bat_2_v
    :field bcn_bat_1_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_bat_1_t
    :field bcn_bat_2_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_bat_2_t
    :field bcn_3p3_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_3p3_i
    :field bcn_cdh_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_cdh_t
    :field bcn_sa_1_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sa_1_v
    :field bcn_sa_1_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sa_1_i
    :field bcn_sa_2_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sa_2_v
    :field bcn_sa_2_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sa_2_i
    :field bcn_eps_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_eps_t
    :field bcn_eps_bus_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_eps_bus_v
    :field bcn_eps_bus_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_eps_bus_i
    :field bcn_xact_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_xact_v
    :field bcn_xact_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_xact_i
    :field bcn_uhf_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_uhf_v
    :field bcn_uhf_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_uhf_i
    :field bcn_sbd_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sbd_v
    :field bcn_sbd_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sbd_i
    :field bcn_vrum_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_vrum_v
    :field bcn_vrum_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_vrum_i
    :field bcn_gps_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_gps_v
    :field bcn_gps_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_gps_i
    :field bcn_boom_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_boom_v
    :field bcn_boom_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_boom_i
    :field bcn_ifb_t_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_ifb_t_1
    :field bcn_adcs_rw_1_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_rw_1_t
    :field bcn_adcs_rw_sp_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_rw_sp_1
    :field bcn_adcs_rw_sp_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_rw_sp_2
    :field bcn_adcs_rw_sp_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_rw_sp_3
    :field bcn_sun_pt_err: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sun_pt_err
    :field bcn_mag_vec_bod_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_mag_vec_bod_1
    :field bcn_mag_vec_bod_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_mag_vec_bod_2
    :field bcn_mag_vec_bod_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_mag_vec_bod_3
    :field bcn_sbd_pa_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sbd_pa_i
    :field bcn_sbd_pa_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sbd_pa_t
    :field bcn_cmd_fail_code: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_cmd_fail_code
    :field bcn_vrum_cmd_rej_rsn: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_vrum_cmd_rej_rsn
    :field bcn_clt_hr_left: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_clt_hr_left
    :field bcn_clt_ct: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_clt_ct
    :field bcn_sys_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_sys_mode
    :field bcn_uhf_alive: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_uhf_alive
    :field reusable_spare_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_2
    :field bcn_pld_pwr_cyc_vrum: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pwr_cyc_vrum
    :field bcn_pld_pwr_off_vrum: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pwr_off_vrum
    :field bcn_pld_stat_st_vrum: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_stat_st_vrum
    :field bcn_pld_time_st_vrum: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_time_st_vrum
    :field bcn_pld_alive_st_vrum: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_alive_st_vrum
    :field reusable_spare_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_3
    :field bcn_pld_pwr_cyc_boom: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pwr_cyc_boom
    :field bcn_pld_pwr_off_boom: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_pwr_off_boom
    :field bcn_pld_stat_st_boom: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_stat_st_boom
    :field bcn_pld_time_st_boom: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_time_st_boom
    :field bcn_pld_alive_st_boom: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pld_alive_st_boom
    :field bcn_uhf_t: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_uhf_t
    :field bcn_seq_state_auto: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_state_auto
    :field bcn_seq_state_op_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_state_op_1
    :field bcn_seq_state_op_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_state_op_2
    :field bcn_seq_state_op_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_seq_state_op_3
    :field reusable_spare_4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_4
    :field reusable_spare_5: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_5
    :field reusable_spare_6: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_6
    :field reusable_spare_7: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_7
    :field reusable_spare_8: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_8
    :field reusable_spare_9: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_9
    :field bcn_htr_pwr_state_bat_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_htr_pwr_state_bat_2
    :field bcn_htr_pwr_state_bat_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_htr_pwr_state_bat_1
    :field reusable_spare_10: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_10
    :field bcn_pwr_state_boom: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_boom
    :field bcn_pwr_state_gps: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_gps
    :field bcn_pwr_state_unused: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_unused
    :field bcn_pwr_state_vrum: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_vrum
    :field bcn_pwr_state_sbd: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_sbd
    :field bcn_pwr_state_uhf: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_uhf
    :field bcn_pwr_state_adcs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_pwr_state_adcs
    :field bcn_fp_task_state: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_task_state
    :field bcn_fp_state_wp_23: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_23
    :field bcn_fp_state_wp_22: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_22
    :field bcn_fp_state_wp_21: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_21
    :field bcn_fp_state_wp_20: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_20
    :field bcn_fp_state_wp_19: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_19
    :field bcn_fp_state_wp_18: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_18
    :field bcn_fp_state_wp_17: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_17
    :field bcn_fp_state_wp_16: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_16
    :field bcn_fp_state_wp_15: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_15
    :field bcn_fp_state_wp_14: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_14
    :field bcn_fp_state_wp_13: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_13
    :field bcn_fp_state_wp_12: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_12
    :field bcn_fp_state_wp_11: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_11
    :field bcn_fp_state_wp_10: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_10
    :field bcn_fp_state_wp_9: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_9
    :field bcn_fp_state_wp_8: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_8
    :field bcn_fp_state_wp_7: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_7
    :field bcn_fp_state_wp_6: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_6
    :field bcn_fp_state_wp_5: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_5
    :field bcn_fp_state_wp_4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_4
    :field bcn_fp_state_wp_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_3
    :field bcn_fp_state_wp_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_2
    :field bcn_fp_state_wp_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_1
    :field bcn_fp_state_wp_0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_fp_state_wp_0
    :field bcn_adcs_alive: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_alive
    :field bcn_adcs_att_vld: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_att_vld
    :field bcn_adcs_ref_vld: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_ref_vld
    :field bcn_adcs_time_vld: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_time_vld
    :field bcn_adcs_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_mode
    :field bcn_adcs_rec_sun_pt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_rec_sun_pt
    :field bcn_adcs_sun_pt_state: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_sun_pt_state
    :field bcn_adcs_cmd_acc: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_cmd_acc
    :field bcn_adcs_cmd_fail: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_cmd_fail
    :field bcn_adcs_cmd_stat: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..bcn_adcs_cmd_stat
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Cosmo.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Cosmo.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Cosmo.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Cosmo.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Cosmo.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Cosmo.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Cosmo.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Cosmo.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Cosmo.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Cosmo.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Cosmo.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Cosmo.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Cosmo.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ascii")
            if not  ((self.callsign == u"LASP  ") or (self.callsign == u"COSMO ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class BeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bcn_adcs_tai_sec = self._io.read_f8be()
            self.bcn_time_since_boot = self._io.read_u4be()
            self.bcn_hr_cyc_ct_safe = self._io.read_u4be()
            self.bcn_sec_in_mode = self._io.read_u4be()
            self.bcn_adcs_bod_rt_1 = self._io.read_s4be()
            self.bcn_adcs_bod_rt_2 = self._io.read_s4be()
            self.bcn_adcs_bod_rt_3 = self._io.read_s4be()
            self.bcn_adcs_att_resid_1 = self._io.read_s4be()
            self.bcn_adcs_att_resid_2 = self._io.read_s4be()
            self.bcn_adcs_att_resid_3 = self._io.read_s4be()
            self.bcn_adcs_gps_pos_ecef_1 = self._io.read_s4be()
            self.bcn_adcs_gps_pos_ecef_2 = self._io.read_s4be()
            self.bcn_adcs_gps_pos_ecef_3 = self._io.read_s4be()
            self.bcn_cmd_rx_ct = self._io.read_u2be()
            self.bcn_cmd_rej_ct = self._io.read_u2be()
            self.bcn_cmd_succ_ct = self._io.read_u2be()
            self.bcn_pld_pass_ct = self._io.read_u2be()
            self.bcn_pld_pass_err = self._io.read_u2be()
            self.bcn_pld_pkt_ct = self._io.read_u2be()
            self.bcn_pld_5_v_0_reg_t = self._io.read_u2be()
            self.bcn_pld_3_v_3_reg_t = self._io.read_u2be()
            self.bcn_pld_sb_1_t = self._io.read_u2be()
            self.bcn_pld_sb_2_t = self._io.read_u2be()
            self.bcn_pld_vrum_t_1 = self._io.read_u2be()
            self.bcn_pld_vrum_t_2 = self._io.read_u2be()
            self.bcn_pld_pkt_rx_ct = self._io.read_u2be()
            self.bcn_pld_pkt_ack_ct = self._io.read_u2be()
            self.bcn_pld_pkt_nack_ct = self._io.read_u2be()
            self.bcn_pld_err_reg_0_0 = self._io.read_u2be()
            self.bcn_pld_err_reg_0_1 = self._io.read_u2be()
            self.bcn_pld_err_reg_1_0 = self._io.read_u2be()
            self.bcn_pld_err_reg_1_1 = self._io.read_u2be()
            self.bcn_seq_exec_buf_auto = self._io.read_u2be()
            self.bcn_seq_exec_buf_op_1 = self._io.read_u2be()
            self.bcn_seq_exec_buf_op_2 = self._io.read_u2be()
            self.bcn_seq_exec_buf_op_3 = self._io.read_u2be()
            self.reusable_spare_1 = self._io.read_u2be()
            self.bcn_store_part_wr_log = self._io.read_u4be()
            self.bcn_store_part_rd_log = self._io.read_u4be()
            self.bcn_store_part_wr_adcs = self._io.read_u4be()
            self.bcn_store_part_rd_adcs = self._io.read_u4be()
            self.bcn_store_part_wr_hk = self._io.read_u4be()
            self.bcn_store_part_rd_hk = self._io.read_u4be()
            self.bcn_store_part_wr_sci = self._io.read_u4be()
            self.bcn_store_part_rd_sci = self._io.read_u4be()
            self.bcn_fp_resp_ct = self._io.read_u2be()
            self.bcn_bat_1_v = self._io.read_u2be()
            self.bcn_bat_2_v = self._io.read_u2be()
            self.bcn_bat_1_t = self._io.read_u2be()
            self.bcn_bat_2_t = self._io.read_u2be()
            self.bcn_3p3_i = self._io.read_u2be()
            self.bcn_cdh_t = self._io.read_u2be()
            self.bcn_sa_1_v = self._io.read_u2be()
            self.bcn_sa_1_i = self._io.read_u2be()
            self.bcn_sa_2_v = self._io.read_u2be()
            self.bcn_sa_2_i = self._io.read_u2be()
            self.bcn_eps_t = self._io.read_u2be()
            self.bcn_eps_bus_v = self._io.read_u2be()
            self.bcn_eps_bus_i = self._io.read_u2be()
            self.bcn_xact_v = self._io.read_u2be()
            self.bcn_xact_i = self._io.read_u2be()
            self.bcn_uhf_v = self._io.read_u2be()
            self.bcn_uhf_i = self._io.read_u2be()
            self.bcn_sbd_v = self._io.read_u2be()
            self.bcn_sbd_i = self._io.read_u2be()
            self.bcn_vrum_v = self._io.read_u2be()
            self.bcn_vrum_i = self._io.read_u2be()
            self.bcn_gps_v = self._io.read_u2be()
            self.bcn_gps_i = self._io.read_u2be()
            self.bcn_boom_v = self._io.read_u2be()
            self.bcn_boom_i = self._io.read_u2be()
            self.bcn_ifb_t_1 = self._io.read_u2be()
            self.bcn_adcs_rw_1_t = self._io.read_s2be()
            self.bcn_adcs_rw_sp_1 = self._io.read_s2be()
            self.bcn_adcs_rw_sp_2 = self._io.read_s2be()
            self.bcn_adcs_rw_sp_3 = self._io.read_s2be()
            self.bcn_sun_pt_err = self._io.read_u2be()
            self.bcn_mag_vec_bod_1 = self._io.read_s2be()
            self.bcn_mag_vec_bod_2 = self._io.read_s2be()
            self.bcn_mag_vec_bod_3 = self._io.read_s2be()
            self.bcn_sbd_pa_i = self._io.read_u2be()
            self.bcn_sbd_pa_t = self._io.read_u2be()
            self.bcn_cmd_fail_code = self._io.read_u1()
            self.bcn_vrum_cmd_rej_rsn = self._io.read_u1()
            self.bcn_clt_hr_left = self._io.read_u1()
            self.bcn_clt_ct = self._io.read_u1()
            self.bcn_sys_mode = self._io.read_u1()
            self.bcn_uhf_alive = self._io.read_u1()
            self.reusable_spare_2 = self._io.read_bits_int_be(2)
            self.bcn_pld_pwr_cyc_vrum = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_pwr_off_vrum = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_stat_st_vrum = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_time_st_vrum = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_alive_st_vrum = self._io.read_bits_int_be(2)
            self.reusable_spare_3 = self._io.read_bits_int_be(2)
            self.bcn_pld_pwr_cyc_boom = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_pwr_off_boom = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_stat_st_boom = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_time_st_boom = self._io.read_bits_int_be(1) != 0
            self.bcn_pld_alive_st_boom = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.bcn_uhf_t = self._io.read_s1()
            self.bcn_seq_state_auto = self._io.read_u1()
            self.bcn_seq_state_op_1 = self._io.read_u1()
            self.bcn_seq_state_op_2 = self._io.read_u1()
            self.bcn_seq_state_op_3 = self._io.read_u1()
            self.reusable_spare_4 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_5 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_6 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_7 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_8 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_9 = self._io.read_bits_int_be(1) != 0
            self.bcn_htr_pwr_state_bat_2 = self._io.read_bits_int_be(1) != 0
            self.bcn_htr_pwr_state_bat_1 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_10 = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_boom = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_gps = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_unused = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_vrum = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_sbd = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_uhf = self._io.read_bits_int_be(1) != 0
            self.bcn_pwr_state_adcs = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.bcn_fp_task_state = self._io.read_u1()
            self.bcn_fp_state_wp_23 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_22 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_21 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_20 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_19 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_18 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_17 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_16 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_15 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_14 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_13 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_12 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_11 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_10 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_9 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_8 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_7 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_6 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_5 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_4 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_3 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_2 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_1 = self._io.read_bits_int_be(2)
            self.bcn_fp_state_wp_0 = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.bcn_adcs_alive = self._io.read_u1()
            self.bcn_adcs_att_vld = self._io.read_bits_int_be(1) != 0
            self.bcn_adcs_ref_vld = self._io.read_bits_int_be(1) != 0
            self.bcn_adcs_time_vld = self._io.read_bits_int_be(1) != 0
            self.bcn_adcs_mode = self._io.read_bits_int_be(1) != 0
            self.bcn_adcs_rec_sun_pt = self._io.read_bits_int_be(1) != 0
            self.bcn_adcs_sun_pt_state = self._io.read_bits_int_be(3)
            self._io.align_to_byte()
            self.bcn_adcs_cmd_acc = self._io.read_u1()
            self.bcn_adcs_cmd_fail = self._io.read_u1()
            self.bcn_adcs_cmd_stat = self._io.read_u1()
            self.padding = self._io.read_u2be()
            self.padding2 = self._io.read_u1()
            self.checksum = self._io.read_u4be()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Cosmo.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class DataSectionT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            if self._parent.packet_primary_header.secondary_header_flag:
                self._raw_secondary_header = self._io.read_bytes(6)
                _io__raw_secondary_header = KaitaiStream(BytesIO(self._raw_secondary_header))
                self.secondary_header = Cosmo.SecondaryHeaderT(_io__raw_secondary_header, self, self._root)

            _on = self._parent.packet_primary_header.application_process_id
            if _on == 28:
                self.user_data_field = Cosmo.CosmoBeaconT(self._io, self, self._root)


    class SecondaryHeaderT(KaitaiStruct):
        """the secondary header is a feature of the space packet which allows
        additional types of information that may be useful to the user
        application (e.g., a time code) to be included.
        see: 4.1.3.2 in ccsds 133.0-b-1
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.time_stamp_seconds = self._io.read_u4be()
            self.sub_seconds = self._io.read_u2be()


    class CosmoBeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_t = Cosmo.BeaconT(self._io, self, self._root)


    class PacketPrimaryHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_version = self._io.read_bits_int_be(3)
            self.packet_type = self._io.read_bits_int_be(1) != 0
            self.secondary_header_flag = self._io.read_bits_int_be(1) != 0
            self.is_stored_data = self._io.read_bits_int_be(1) != 0
            self.application_process_id = self._io.read_bits_int_be(10)
            self.grouping_flag = self._io.read_bits_int_be(2)
            self.sequence_count = self._io.read_bits_int_be(14)
            self._io.align_to_byte()
            self.packet_length = self._io.read_u2be()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Cosmo.Callsign(_io__raw_callsign_ror, self, self._root)


    class CcsdsSpacePacketT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_packet_primary_header = self._io.read_bytes(6)
            _io__raw_packet_primary_header = KaitaiStream(BytesIO(self._raw_packet_primary_header))
            self.packet_primary_header = Cosmo.PacketPrimaryHeaderT(_io__raw_packet_primary_header, self, self._root)
            self.data_section = Cosmo.DataSectionT(self._io, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_space_packet = Cosmo.CcsdsSpacePacketT(self._io, self, self._root)






"""
SatNOGS Decoder subpackage initialization
"""
from __future__ import absolute_import, division, print_function

import enum
import functools
import importlib
import os
import re

from kaitaistruct import KaitaiStruct

# Add all decoders automatically to the current namespace
for file in os.listdir(os.path.dirname(__file__)):
    if not file.endswith('.py') or file.startswith('__'):
        continue
    module_name = file[:-3]

    module = importlib.import_module(f'.{module_name}', package=__package__)
    class_name = module_name.capitalize().replace('_', '')

    try:
        globals()[class_name] = getattr(module, class_name)
    except AttributeError as ex:
        print(f'WARNING: Import of decoder class {class_name} failed.\n'
              ' Did you compile modified Kaitai structs to Python code yet? \n'
              f'AttributeError: {ex}')

FIELD_REGEX = re.compile(
    r':field (?P<field>[\*\w]+): (?P<attribute>.*?)'
    r'(?:(?=:field)|\Z|\n)', re.S)

UNKNOWN_SIZE_NOTATION = '___'


def get_attribute(obj, name):
    """
    Get element by index in case of list
    Get attribute of object by namein case of non-list object
    """
    if isinstance(obj, list):
        try:
            return obj[int(name)]
        except ValueError:
            return getattr(obj, name).name
    if isinstance(getattr(obj, name), enum.Enum):
        return getattr(obj, name).name
    return getattr(obj, name)


def get_dynamic_fields(obj, path_list, indexes, inframe_index, key):
    """
    Get element by recusion in case of unknown sized list
    """
    if inframe_index == len(path_list):
        pos = 1
        for k in indexes:
            key = key.replace(UNKNOWN_SIZE_NOTATION,
                              '_' + str(indexes[k][0] - 1) + '_', pos)
            pos += 1

        return obj, key

    name = path_list[inframe_index]
    if isinstance(obj, list):
        try:
            if name == UNKNOWN_SIZE_NOTATION:
                index_key = indexes[path_list[inframe_index - 1]][0]
                indexes[path_list[inframe_index - 1]][0] += 1
                if indexes[path_list[inframe_index - 1]][1] is None:
                    indexes[path_list[inframe_index - 1]][1] = len(obj)
                return get_dynamic_fields(obj[index_key], path_list, indexes,
                                          inframe_index + 1, key)

            return get_dynamic_fields(obj[int(name)], path_list, indexes,
                                      inframe_index + 1, key)
        except ValueError:
            return get_dynamic_fields(
                getattr(obj, name).name, path_list, indexes, inframe_index + 1,
                key)
    if isinstance(getattr(obj, name), enum.Enum):
        return get_dynamic_fields(
            getattr(obj, name).name, path_list, indexes, inframe_index + 1,
            key)
    return get_dynamic_fields(getattr(obj, name), path_list, indexes,
                              inframe_index + 1, key)


def get_fields(struct: KaitaiStruct, empty=False):
    """
    Extract fields from Kaitai Struct as defined in the Struct docstring
    and return as dictionary.

    Args:
        struct: Satellite Decoder object
        empty (bool): If True, fields with invalid paths get None value.
                      If False, fields with invalid paths are omitted.

    Returns:
        dict: Field values mapped to field names
    """
    fields = {}
    dynamic_fields = {}

    try:
        doc_fields = FIELD_REGEX.findall(struct.__doc__)
    except TypeError:
        return fields

    for key, value in doc_fields:
        try:
            if UNKNOWN_SIZE_NOTATION not in key:
                fields[key] = functools.reduce(get_attribute, value.split('.'),
                                               struct)
            else:
                key_values = key.split(UNKNOWN_SIZE_NOTATION)
                for i in range(0, len(key_values) - 1):
                    dynamic_fields[key_values[i]] = [0, None]

                path_list = value.split('.')
                while True:
                    value, generated_key = get_dynamic_fields(
                        struct, path_list, dynamic_fields, 0, key)
                    fields[generated_key] = value
                    first_dynamic_field_key = next(iter(dynamic_fields))
                    if dynamic_fields[first_dynamic_field_key][
                            0] == dynamic_fields[first_dynamic_field_key][1]:
                        break

        except (AttributeError, IndexError):
            if empty:
                fields[key] = None

    return fields


def kaitai_to_dict(struct):
    """
    Convert a Kaitai Struct parsed object to a nested dictionary.
    Handles nested objects, arrays, and primitive types.

    Args:
        struct: A Kaitai Struct parsed object

    Returns:
        dict: A nested dictionary representation of the Kaitai object / frame
    """
    if isinstance(struct, (int, float, str, bool)) or struct is None:
        return struct
    if isinstance(struct, bytes):
        return struct.hex()
    if isinstance(struct, list):
        return [kaitai_to_dict(item) for item in struct]
    if hasattr(struct, '__dict__'):
        result = {}
        # Get all public attributes (not starting with '_')
        for key in dir(struct):
            # Skip private attributes
            if key.startswith('_'):
                continue

            value = getattr(struct, key)

            # Skip callable methods
            if callable(value):
                continue

            result[key] = kaitai_to_dict(value)
        return result
    raise ValueError('struct must be a valid kaitai struct'
                     'or recursive sub-member.')



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Hype(KaitaiStruct):
    """:field service: service
    :field callsign: callsign
    :field counter: counter
    :field sat_mode: sat_mode
    :field vbus: vbus
    :field balance: balance
    :field charge: charge
    :field sat_temp: sat_temp
    :field sw_ver: sw_ver
    :field restart_counter: restart_counter
    :field uptime: uptime
    :field x_rate: x_rate
    :field y_rate: y_rate
    :field z_rate: z_rate
    :field sys_status: sys_status
    :field sys_status_dbm: sys_status_dbm
    :field sys_status_reserved: sys_status_reserved
    :field sys_status_software_slot_selected: sys_status_software_slot_selected
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.service = self._io.read_u1()
        self.callsign = (self._io.read_bytes(4)).decode(u"ASCII")
        if not self.callsign == u"HYPE":
            raise kaitaistruct.ValidationNotEqualError(u"HYPE", self.callsign, self._io, u"/seq/1")
        self.counter = self._io.read_u4be()
        self.sat_mode = self._io.read_u1()
        self.vbus = self._io.read_u2be()
        self.balance_raw = self._io.read_s2be()
        self.charge_raw = self._io.read_u2be()
        self.sat_temp = self._io.read_s2be()
        self.sw_ver = self._io.read_u2be()
        self.restart_counter = self._io.read_u4be()
        self.uptime = self._io.read_u4be()
        self.x_rate_raw = self._io.read_s2be()
        self.y_rate_raw = self._io.read_s2be()
        self.z_rate_raw = self._io.read_s2be()
        self.sys_status = self._io.read_u1()

    @property
    def sys_status_dbm(self):
        if hasattr(self, '_m_sys_status_dbm'):
            return self._m_sys_status_dbm

        self._m_sys_status_dbm = (self.sys_status & 63)
        return getattr(self, '_m_sys_status_dbm', None)

    @property
    def charge(self):
        if hasattr(self, '_m_charge'):
            return self._m_charge

        if self.sat_mode != 0:
            self._m_charge = self.charge_raw

        return getattr(self, '_m_charge', None)

    @property
    def z_rate(self):
        if hasattr(self, '_m_z_rate'):
            return self._m_z_rate

        if self.sat_mode != 0:
            self._m_z_rate = self.z_rate_raw

        return getattr(self, '_m_z_rate', None)

    @property
    def y_rate(self):
        if hasattr(self, '_m_y_rate'):
            return self._m_y_rate

        if self.sat_mode != 0:
            self._m_y_rate = self.y_rate_raw

        return getattr(self, '_m_y_rate', None)

    @property
    def sys_status_reserved(self):
        if hasattr(self, '_m_sys_status_reserved'):
            return self._m_sys_status_reserved

        self._m_sys_status_reserved = ((self.sys_status & 64) >> 6)
        return getattr(self, '_m_sys_status_reserved', None)

    @property
    def balance(self):
        if hasattr(self, '_m_balance'):
            return self._m_balance

        if self.sat_mode != 0:
            self._m_balance = self.balance_raw

        return getattr(self, '_m_balance', None)

    @property
    def sys_status_software_slot_selected(self):
        if hasattr(self, '_m_sys_status_software_slot_selected'):
            return self._m_sys_status_software_slot_selected

        self._m_sys_status_software_slot_selected = ((self.sys_status & 128) >> 7)
        return getattr(self, '_m_sys_status_software_slot_selected', None)

    @property
    def x_rate(self):
        if hasattr(self, '_m_x_rate'):
            return self._m_x_rate

        if self.sat_mode != 0:
            self._m_x_rate = self.x_rate_raw

        return getattr(self, '_m_x_rate', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Bdsat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field obc_pass_packet_type: ax25_frame.payload.ax25_info.body.obc_pass_packet_type
    :field obc_rst_cnt_str: ax25_frame.payload.ax25_info.body.obc_rst_cnt_str
    :field obc_uptime_str: ax25_frame.payload.ax25_info.body.obc_uptime_str
    :field obc_bat_str: ax25_frame.payload.ax25_info.body.obc_bat_str
    :field obc_temp_obc_str: ax25_frame.payload.ax25_info.body.obc_temp_obc_str
    :field obc_temp_zn_str: ax25_frame.payload.ax25_info.body.obc_temp_zn_str
    :field obc_temp_xp_str: ax25_frame.payload.ax25_info.body.obc_temp_xp_str
    :field obc_temp_yp_str: ax25_frame.payload.ax25_info.body.obc_temp_yp_str
    :field obc_temp_yn_str: ax25_frame.payload.ax25_info.body.obc_temp_yn_str
    :field obc_temp_xn_str: ax25_frame.payload.ax25_info.body.obc_temp_xn_str
    :field obc_freemem_str: ax25_frame.payload.ax25_info.body.obc_freemem_str
    :field obc_pld_state_str: ax25_frame.payload.ax25_info.body.obc_pld_state_str
    :field obc_pld_prog_str: ax25_frame.payload.ax25_info.body.obc_pld_prog_str
    :field obc_pld_hw_state_str: ax25_frame.payload.ax25_info.body.obc_pld_hw_state_str
    :field obc_psu_uptime_str: ax25_frame.payload.ax25_info.body.obc_psu_uptime_str
    :field obc_crc_str: ax25_frame.payload.ax25_info.body.obc_crc_str
    :field obc_reset_cnt: ax25_frame.payload.ax25_info.body.obc_reset_cnt
    :field obc_uptime: ax25_frame.payload.ax25_info.body.obc_uptime
    :field obc_bat: ax25_frame.payload.ax25_info.body.obc_bat
    :field obc_temp_obc: ax25_frame.payload.ax25_info.body.obc_temp_obc
    :field obc_temp_zn: ax25_frame.payload.ax25_info.body.obc_temp_zn
    :field obc_temp_xp: ax25_frame.payload.ax25_info.body.obc_temp_xp
    :field obc_temp_yp: ax25_frame.payload.ax25_info.body.obc_temp_yp
    :field obc_temp_yn: ax25_frame.payload.ax25_info.body.obc_temp_yn
    :field obc_temp_xn: ax25_frame.payload.ax25_info.body.obc_temp_xn
    :field obc_freemem: ax25_frame.payload.ax25_info.body.obc_freemem
    :field obc_pld_state: ax25_frame.payload.ax25_info.body.obc_pld_state
    :field obc_pld_prog: ax25_frame.payload.ax25_info.body.obc_pld_prog
    :field obc_pld_hw_state: ax25_frame.payload.ax25_info.body.obc_pld_hw_state
    :field obc_psu_uptime: ax25_frame.payload.ax25_info.body.obc_psu_uptime
    :field trx_uptime_str: ax25_frame.payload.ax25_info.body.trx_uptime_str
    :field trx_uptime_total_str: ax25_frame.payload.ax25_info.body.trx_uptime_total_str
    :field trx_reset_cnt_str: ax25_frame.payload.ax25_info.body.trx_reset_cnt_str
    :field trx_trx_temp_str: ax25_frame.payload.ax25_info.body.trx_trx_temp_str
    :field trx_rf_temp_str: ax25_frame.payload.ax25_info.body.trx_rf_temp_str
    :field trx_pa_temp_str: ax25_frame.payload.ax25_info.body.trx_pa_temp_str
    :field trx_digipeater_cnt_str: ax25_frame.payload.ax25_info.body.trx_digipeater_cnt_str
    :field trx_last_digipeater_str: ax25_frame.payload.ax25_info.body.trx_last_digipeater_str
    :field trx_rx_cnt_str: ax25_frame.payload.ax25_info.body.trx_rx_cnt_str
    :field trx_tx_cnt_str: ax25_frame.payload.ax25_info.body.trx_tx_cnt_str
    :field trx_act_rssi_raw_str: ax25_frame.payload.ax25_info.body.trx_act_rssi_raw_str
    :field trx_dcd_rssi_raw_str: ax25_frame.payload.ax25_info.body.trx_dcd_rssi_raw_str
    :field trx_uptime: ax25_frame.payload.ax25_info.body.trx_uptime
    :field trx_uptime_total: ax25_frame.payload.ax25_info.body.trx_uptime_total
    :field trx_reset_cnt: ax25_frame.payload.ax25_info.body.trx_reset_cnt
    :field trx_trx_temp: ax25_frame.payload.ax25_info.body.trx_trx_temp
    :field trx_rf_temp: ax25_frame.payload.ax25_info.body.trx_rf_temp
    :field trx_pa_temp: ax25_frame.payload.ax25_info.body.trx_pa_temp
    :field trx_digipeater_cnt: ax25_frame.payload.ax25_info.body.trx_digipeater_cnt
    :field trx_last_digipeater: ax25_frame.payload.ax25_info.body.trx_last_digipeater
    :field trx_rx_cnt: ax25_frame.payload.ax25_info.body.trx_rx_cnt
    :field trx_tx_cnt: ax25_frame.payload.ax25_info.body.trx_tx_cnt
    :field trx_act_rssi_raw: ax25_frame.payload.ax25_info.body.trx_act_rssi_raw
    :field trx_dcd_rssi_raw: ax25_frame.payload.ax25_info.body.trx_dcd_rssi_raw
    :field monitor: ax25_frame.payload.ax25_info.monitor
    :field packet_type_q: ax25_frame.payload.ax25_info.packet_type_q
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Bdsat.Ax25Frame(self._io, self, self._root)

    class Trx(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.trx_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_uptime_total_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_reset_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_trx_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_rf_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_pa_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_digipeater_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_last_digipeater_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_rx_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_tx_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_act_rssi_raw_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.trx_dcd_rssi_raw_str = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf8")

        @property
        def trx_act_rssi_raw(self):
            if hasattr(self, '_m_trx_act_rssi_raw'):
                return self._m_trx_act_rssi_raw

            self._m_trx_act_rssi_raw = int(self.trx_act_rssi_raw_str)
            return getattr(self, '_m_trx_act_rssi_raw', None)

        @property
        def trx_uptime(self):
            if hasattr(self, '_m_trx_uptime'):
                return self._m_trx_uptime

            self._m_trx_uptime = int(self.trx_uptime_str)
            return getattr(self, '_m_trx_uptime', None)

        @property
        def trx_uptime_total(self):
            if hasattr(self, '_m_trx_uptime_total'):
                return self._m_trx_uptime_total

            self._m_trx_uptime_total = int(self.trx_uptime_total_str)
            return getattr(self, '_m_trx_uptime_total', None)

        @property
        def trx_last_digipeater(self):
            if hasattr(self, '_m_trx_last_digipeater'):
                return self._m_trx_last_digipeater

            self._m_trx_last_digipeater = self.trx_last_digipeater_str
            return getattr(self, '_m_trx_last_digipeater', None)

        @property
        def trx_rf_temp(self):
            if hasattr(self, '_m_trx_rf_temp'):
                return self._m_trx_rf_temp

            self._m_trx_rf_temp = int(self.trx_rf_temp_str)
            return getattr(self, '_m_trx_rf_temp', None)

        @property
        def trx_tx_cnt(self):
            if hasattr(self, '_m_trx_tx_cnt'):
                return self._m_trx_tx_cnt

            self._m_trx_tx_cnt = int(self.trx_tx_cnt_str)
            return getattr(self, '_m_trx_tx_cnt', None)

        @property
        def trx_digipeater_cnt(self):
            if hasattr(self, '_m_trx_digipeater_cnt'):
                return self._m_trx_digipeater_cnt

            self._m_trx_digipeater_cnt = int(self.trx_digipeater_cnt_str)
            return getattr(self, '_m_trx_digipeater_cnt', None)

        @property
        def trx_rx_cnt(self):
            if hasattr(self, '_m_trx_rx_cnt'):
                return self._m_trx_rx_cnt

            self._m_trx_rx_cnt = int(self.trx_rx_cnt_str)
            return getattr(self, '_m_trx_rx_cnt', None)

        @property
        def trx_reset_cnt(self):
            if hasattr(self, '_m_trx_reset_cnt'):
                return self._m_trx_reset_cnt

            self._m_trx_reset_cnt = int(self.trx_reset_cnt_str)
            return getattr(self, '_m_trx_reset_cnt', None)

        @property
        def trx_trx_temp(self):
            if hasattr(self, '_m_trx_trx_temp'):
                return self._m_trx_trx_temp

            self._m_trx_trx_temp = int(self.trx_trx_temp_str)
            return getattr(self, '_m_trx_trx_temp', None)

        @property
        def trx_dcd_rssi_raw(self):
            if hasattr(self, '_m_trx_dcd_rssi_raw'):
                return self._m_trx_dcd_rssi_raw

            self._m_trx_dcd_rssi_raw = int(self.trx_dcd_rssi_raw_str)
            return getattr(self, '_m_trx_dcd_rssi_raw', None)

        @property
        def trx_pa_temp(self):
            if hasattr(self, '_m_trx_pa_temp'):
                return self._m_trx_pa_temp

            self._m_trx_pa_temp = int(self.trx_pa_temp_str)
            return getattr(self, '_m_trx_pa_temp', None)


    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Bdsat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Bdsat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Bdsat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Bdsat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Bdsat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Bdsat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Bdsat.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Bdsat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Bdsat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Bdsat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Bdsat.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Bdsat.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Bdsat.Tlm(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Bdsat.Tlm(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Bdsat.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Bdsat.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Bdsat.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class Obc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.obc_pass_packet_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_rst_cnt_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_bat_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_obc_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_zn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_xp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_yp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_yn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_temp_xn_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_freemem_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_pld_state_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_pld_prog_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_pld_hw_state_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_psu_uptime_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.obc_crc_str = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf8")

        @property
        def obc_pld_hw_state(self):
            if hasattr(self, '_m_obc_pld_hw_state'):
                return self._m_obc_pld_hw_state

            self._m_obc_pld_hw_state = int(self.obc_pld_hw_state_str)
            return getattr(self, '_m_obc_pld_hw_state', None)

        @property
        def obc_uptime(self):
            if hasattr(self, '_m_obc_uptime'):
                return self._m_obc_uptime

            self._m_obc_uptime = int(self.obc_uptime_str)
            return getattr(self, '_m_obc_uptime', None)

        @property
        def obc_temp_xp(self):
            if hasattr(self, '_m_obc_temp_xp'):
                return self._m_obc_temp_xp

            self._m_obc_temp_xp = int(self.obc_temp_xp_str)
            return getattr(self, '_m_obc_temp_xp', None)

        @property
        def obc_psu_uptime(self):
            if hasattr(self, '_m_obc_psu_uptime'):
                return self._m_obc_psu_uptime

            self._m_obc_psu_uptime = int(self.obc_psu_uptime_str)
            return getattr(self, '_m_obc_psu_uptime', None)

        @property
        def obc_temp_yp(self):
            if hasattr(self, '_m_obc_temp_yp'):
                return self._m_obc_temp_yp

            self._m_obc_temp_yp = int(self.obc_temp_yp_str)
            return getattr(self, '_m_obc_temp_yp', None)

        @property
        def obc_freemem(self):
            if hasattr(self, '_m_obc_freemem'):
                return self._m_obc_freemem

            self._m_obc_freemem = int(self.obc_freemem_str)
            return getattr(self, '_m_obc_freemem', None)

        @property
        def obc_pld_prog(self):
            if hasattr(self, '_m_obc_pld_prog'):
                return self._m_obc_pld_prog

            self._m_obc_pld_prog = int(self.obc_pld_prog_str)
            return getattr(self, '_m_obc_pld_prog', None)

        @property
        def obc_temp_zn(self):
            if hasattr(self, '_m_obc_temp_zn'):
                return self._m_obc_temp_zn

            self._m_obc_temp_zn = int(self.obc_temp_zn_str)
            return getattr(self, '_m_obc_temp_zn', None)

        @property
        def obc_temp_obc(self):
            if hasattr(self, '_m_obc_temp_obc'):
                return self._m_obc_temp_obc

            self._m_obc_temp_obc = int(self.obc_temp_obc_str)
            return getattr(self, '_m_obc_temp_obc', None)

        @property
        def obc_bat(self):
            if hasattr(self, '_m_obc_bat'):
                return self._m_obc_bat

            self._m_obc_bat = int(self.obc_bat_str)
            return getattr(self, '_m_obc_bat', None)

        @property
        def obc_reset_cnt(self):
            if hasattr(self, '_m_obc_reset_cnt'):
                return self._m_obc_reset_cnt

            self._m_obc_reset_cnt = int(self.obc_rst_cnt_str)
            return getattr(self, '_m_obc_reset_cnt', None)

        @property
        def obc_temp_yn(self):
            if hasattr(self, '_m_obc_temp_yn'):
                return self._m_obc_temp_yn

            self._m_obc_temp_yn = int(self.obc_temp_yn_str)
            return getattr(self, '_m_obc_temp_yn', None)

        @property
        def obc_temp_xn(self):
            if hasattr(self, '_m_obc_temp_xn'):
                return self._m_obc_temp_xn

            self._m_obc_temp_xn = int(self.obc_temp_xn_str)
            return getattr(self, '_m_obc_temp_xn', None)

        @property
        def obc_pld_state(self):
            if hasattr(self, '_m_obc_pld_state'):
                return self._m_obc_pld_state

            self._m_obc_pld_state = int(self.obc_pld_state_str)
            return getattr(self, '_m_obc_pld_state', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Bdsat.Callsign(_io__raw_callsign_ror, self, self._root)


    class Tlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.packet_type_q
            if _on == 79:
                self.body = Bdsat.Obc(self._io, self, self._root)
            else:
                self.body = Bdsat.Trx(self._io, self, self._root)

        @property
        def monitor(self):
            if hasattr(self, '_m_monitor'):
                return self._m_monitor

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_monitor = (self._io.read_bytes_full()).decode(u"ASCII")
            self._io.seek(_pos)
            return getattr(self, '_m_monitor', None)

        @property
        def packet_type_q(self):
            if hasattr(self, '_m_packet_type_q'):
                return self._m_packet_type_q

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_packet_type_q = self._io.read_u1()
            self._io.seek(_pos)
            return getattr(self, '_m_packet_type_q', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Diy1(KaitaiStruct):
    """:field timestamp: diy1_frame.timestamp
    :field nn1: diy1_frame.nn1
    :field nn2: diy1_frame.nn2
    :field nn3: diy1_frame.nn3
    :field nn4: diy1_frame.nn4
    :field nn5: diy1_frame.nn5
    :field nn6: diy1_frame.nn6
    :field nn7: diy1_frame.nn7
    :field nn8: diy1_frame.nn8
    :field nn9: diy1_frame.nn9
    :field ssb: diy1_frame.ssb
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.diy1_frame = Diy1.Diy1FrameT(self._io, self, self._root)

    class Diy1FrameT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.control = self._io.read_bytes(5)
            self.sentence_init = (self._io.read_bytes(1)).decode(u"ASCII")
            if not self.sentence_init == u"$":
                raise kaitaistruct.ValidationNotEqualError(u"$", self.sentence_init, self._io, u"/types/diy1_frame_t/seq/1")
            self.ts_hh_str = (self._io.read_bytes(2)).decode(u"ASCII")
            self.ts_mm_str = (self._io.read_bytes(2)).decode(u"ASCII")
            self.ts_ss_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn1_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn2_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn3_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn4_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn5_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn6_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn7_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn8_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.nn9_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.ssb_raw = self._io.read_bytes(2)
            self.sentence_end = (self._io.read_bytes(1)).decode(u"ASCII")
            if not self.sentence_end == u"*":
                raise kaitaistruct.ValidationNotEqualError(u"*", self.sentence_end, self._io, u"/types/diy1_frame_t/seq/15")

        @property
        def ssb(self):
            """status bits hex numbers
            bit state                          state
            0     1    = PA Mediam Power         0    = PA High Power (bit 5 = 0) OPERATIVE MODE
            1     1    = ROBOT logger full       0    = logger ROBOT free
            2     1    = ROBOT CALLSIGN change   0    = Robot CALLSIGN not change
            3     1    = ROBOT OP ON             0    = ROBOT OP OFF
            4     1    = logger full             0    = logger empty
            5     1    = PA Low Power            0    = disable  (bit 1 = 0) RECOVERY MODE
            6     1    = RTC setted              0    = RTC no setted
            7     1    = command received        0    = command not received
            """
            if hasattr(self, '_m_ssb'):
                return self._m_ssb

            self._m_ssb = (((KaitaiStream.byte_array_index(self.ssb_raw, 0) - 48) * 16) + (KaitaiStream.byte_array_index(self.ssb_raw, 1) - 48))
            return getattr(self, '_m_ssb', None)

        @property
        def nn1(self):
            """solar charger current [mA]."""
            if hasattr(self, '_m_nn1'):
                return self._m_nn1

            self._m_nn1 = int(self.nn1_str)
            return getattr(self, '_m_nn1', None)

        @property
        def nn6(self):
            """RSSI value [dBm]."""
            if hasattr(self, '_m_nn6'):
                return self._m_nn6

            self._m_nn6 = int(self.nn6_str)
            return getattr(self, '_m_nn6', None)

        @property
        def nn4(self):
            """transceiver current in TX mode [mA]."""
            if hasattr(self, '_m_nn4'):
                return self._m_nn4

            self._m_nn4 = int(self.nn4_str)
            return getattr(self, '_m_nn4', None)

        @property
        def nn5(self):
            """battery voltage [cV]."""
            if hasattr(self, '_m_nn5'):
                return self._m_nn5

            self._m_nn5 = int(self.nn5_str)
            return getattr(self, '_m_nn5', None)

        @property
        def nn7(self):
            """OBC temperature [cdegC]."""
            if hasattr(self, '_m_nn7'):
                return self._m_nn7

            self._m_nn7 = int(self.nn7_str)
            return getattr(self, '_m_nn7', None)

        @property
        def nn8(self):
            """temperature of transceiver PA [cdegC]."""
            if hasattr(self, '_m_nn8'):
                return self._m_nn8

            self._m_nn8 = int(self.nn8_str)
            return getattr(self, '_m_nn8', None)

        @property
        def nn9(self):
            """resets numbers."""
            if hasattr(self, '_m_nn9'):
                return self._m_nn9

            self._m_nn9 = int(self.nn9_str)
            return getattr(self, '_m_nn9', None)

        @property
        def timestamp(self):
            """time since last reset (DL4PD: in [s])."""
            if hasattr(self, '_m_timestamp'):
                return self._m_timestamp

            self._m_timestamp = ((((int(self.ts_hh_str) * 60) * 60) + (int(self.ts_mm_str) * 60)) + int(self.ts_ss_str))
            return getattr(self, '_m_timestamp', None)

        @property
        def nn2(self):
            """logic current [mA]."""
            if hasattr(self, '_m_nn2'):
                return self._m_nn2

            self._m_nn2 = int(self.nn2_str)
            return getattr(self, '_m_nn2', None)

        @property
        def nn3(self):
            """transceiver current in RX mode [mA]."""
            if hasattr(self, '_m_nn3'):
                return self._m_nn3

            self._m_nn3 = int(self.nn3_str)
            return getattr(self, '_m_nn3', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Silversat(KaitaiStruct):
    """:field cw_callsign: silversat.cw_or_ssdv.cw_callsign
    :field power: silversat.cw_or_ssdv.power
    :field avionics: silversat.cw_or_ssdv.avionics
    :field payload: silversat.cw_or_ssdv.payload
    :field radio: silversat.cw_or_ssdv.radio
    :field cw_beacon: silversat.cw_or_ssdv.cw_beacon
    :field dest_callsign: silversat.cw_or_ssdv.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: silversat.cw_or_ssdv.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: silversat.cw_or_ssdv.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: silversat.cw_or_ssdv.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: silversat.cw_or_ssdv.ax25_frame.ax25_header.ctl
    :field pid: silversat.cw_or_ssdv.ax25_frame.ax25_header.pid
    :field ssdv_sync_byte: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_sync_byte
    :field ssdv_packet_type: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_packet_type
    :field ssdv_callsign: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_callsign
    :field ssdv_image_id: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_image_id
    :field ssdv_packet_id: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_packet_id
    :field ssdv_width: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_width
    :field ssdv_height: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_height
    :field ssdv_flags: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_flags
    :field ssdv_mcu_offset: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_mcu_offset
    :field ssdv_mcu_index: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_mcu_index
    :field ssdv_image: silversat.cw_or_ssdv.ax25_frame.ax25_info.ssdv_image
    
    .. seealso::
       Source - https://github.com/silver-sat/systems/wiki/Beacon-Operations
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.silversat = Silversat.SilversatT(self._io, self, self._root)

    class SilversatT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.cw_or_ssdv_switch_on
            if _on == 2003841656:
                self.cw_or_ssdv = Silversat.Cw(self._io, self, self._root)
            else:
                self.cw_or_ssdv = Silversat.Ssdv(self._io, self, self._root)

        @property
        def cw_or_ssdv_switch_on(self):
            if hasattr(self, '_m_cw_or_ssdv_switch_on'):
                return self._m_cw_or_ssdv_switch_on

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_cw_or_ssdv_switch_on = self._io.read_u4be()
            self._io.seek(_pos)
            return getattr(self, '_m_cw_or_ssdv_switch_on', None)


    class Cw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cw_callsign = (self._io.read_bytes(7)).decode(u"ASCII")
            if not self.cw_callsign == u"wp2xgw ":
                raise kaitaistruct.ValidationNotEqualError(u"wp2xgw ", self.cw_callsign, self._io, u"/types/cw/seq/0")
            self.power = (self._io.read_bytes(1)).decode(u"ASCII")
            if not  ((self.power == u"s") or (self.power == u"e") or (self.power == u"i") or (self.power == u"t") or (self.power == u"a")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.power, self._io, u"/types/cw/seq/1")
            self.avionics = (self._io.read_bytes(1)).decode(u"ASCII")
            if not  ((self.avionics == u"e") or (self.avionics == u"s") or (self.avionics == u"a") or (self.avionics == u"h") or (self.avionics == u"n") or (self.avionics == u"u") or (self.avionics == u"i") or (self.avionics == u"d") or (self.avionics == u"r") or (self.avionics == u"t") or (self.avionics == u"5")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.avionics, self._io, u"/types/cw/seq/2")
            self.payload = (self._io.read_bytes(1)).decode(u"ASCII")
            if not  ((self.payload == u"e") or (self.payload == u"i") or (self.payload == u"v") or (self.payload == u"w") or (self.payload == u"l") or (self.payload == u"5") or (self.payload == u"a") or (self.payload == u"h") or (self.payload == u"m") or (self.payload == u"4") or (self.payload == u"6") or (self.payload == u"c") or (self.payload == u"b") or (self.payload == u"s") or (self.payload == u"t") or (self.payload == u"n") or (self.payload == u"r") or (self.payload == u"u") or (self.payload == u"d") or (self.payload == u"f") or (self.payload == u"g") or (self.payload == u"k") or (self.payload == u"o") or (self.payload == u"7") or (self.payload == u"x") or (self.payload == u"z") or (self.payload == u"p") or (self.payload == u"3") or (self.payload == u"j") or (self.payload == u"q") or (self.payload == u"9") or (self.payload == u"0")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.payload, self._io, u"/types/cw/seq/3")
            self.radio = (self._io.read_bytes(1)).decode(u"ASCII")
            if not  ((self.radio == u"e") or (self.radio == u"i") or (self.radio == u"s") or (self.radio == u"t") or (self.radio == u"n") or (self.radio == u"a") or (self.radio == u"h") or (self.radio == u"d") or (self.radio == u"r") or (self.radio == u"u") or (self.radio == u"5") or (self.radio == u"b") or (self.radio == u"v") or (self.radio == u"f") or (self.radio == u"l") or (self.radio == u"m") or (self.radio == u"4") or (self.radio == u"g")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.radio, self._io, u"/types/cw/seq/4")
            self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.lengthcheck) != 0:
                self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def cw_beacon(self):
            if hasattr(self, '_m_cw_beacon'):
                return self._m_cw_beacon

            self._m_cw_beacon = self.power + self.avionics + self.payload + self.radio
            return getattr(self, '_m_cw_beacon', None)


    class Ssdv(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Silversat.Ssdv.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Silversat.Ssdv.Ax25Header(self._io, self, self._root)
                self.ax25_info = Silversat.Ssdv.SsdvPayload(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Silversat.Ssdv.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Silversat.Ssdv.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Silversat.Ssdv.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Silversat.Ssdv.SsidMask(self._io, self, self._root)
                self.ctl = self._io.read_u1()
                self.pid = self._io.read_u1()


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class SsdvPayload(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssdv_sync_byte = self._io.read_u1()
                if not self.ssdv_sync_byte == 85:
                    raise kaitaistruct.ValidationNotEqualError(85, self.ssdv_sync_byte, self._io, u"/types/ssdv/types/ssdv_payload/seq/0")
                self.ssdv_packet_type = self._io.read_u1()
                if not  ((self.ssdv_packet_type == 102) or (self.ssdv_packet_type == 103)) :
                    raise kaitaistruct.ValidationNotAnyOfError(self.ssdv_packet_type, self._io, u"/types/ssdv/types/ssdv_payload/seq/1")
                self.ssdv_callsign = self._io.read_u4be()
                if not self.ssdv_callsign == 3739973996:
                    raise kaitaistruct.ValidationNotEqualError(3739973996, self.ssdv_callsign, self._io, u"/types/ssdv/types/ssdv_payload/seq/2")
                self.ssdv_image_id = self._io.read_u1()
                self.ssdv_packet_id = self._io.read_u2be()
                self.ssdv_width = self._io.read_u1()
                self.ssdv_height = self._io.read_u1()
                self.ssdv_flags = self._io.read_u1()
                self.ssdv_mcu_offset = self._io.read_u1()
                self.ssdv_mcu_index = self._io.read_u2be()
                self.ssdv_image = []
                i = 0
                while not self._io.is_eof():
                    self.ssdv_image.append(self._io.read_u1())
                    i += 1



        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Silversat.Ssdv.Callsign(_io__raw_callsign_ror, self, self._root)







# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Siriussat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field beacon_type: ax25_frame.payload.ax25_info.beacon_type
    :field usb1: ax25_frame.payload.ax25_info.body.usb1
    :field usb2: ax25_frame.payload.ax25_info.body.usb2
    :field usb3: ax25_frame.payload.ax25_info.body.usb3
    :field isb1: ax25_frame.payload.ax25_info.body.isb1
    :field isb2: ax25_frame.payload.ax25_info.body.isb2
    :field isb3: ax25_frame.payload.ax25_info.body.isb3
    :field iab: ax25_frame.payload.ax25_info.body.iab
    :field ich1: ax25_frame.payload.ax25_info.body.ich1
    :field ich2: ax25_frame.payload.ax25_info.body.ich2
    :field ich3: ax25_frame.payload.ax25_info.body.ich3
    :field ich4: ax25_frame.payload.ax25_info.body.ich4
    :field t1_pw: ax25_frame.payload.ax25_info.body.t1_pw
    :field t2_pw: ax25_frame.payload.ax25_info.body.t2_pw
    :field t3_pw: ax25_frame.payload.ax25_info.body.t3_pw
    :field t4_pw: ax25_frame.payload.ax25_info.body.t4_pw
    :field uab_crit: ax25_frame.payload.ax25_info.body.uab_crit
    :field uab_min: ax25_frame.payload.ax25_info.body.uab_min
    :field heater2_manual: ax25_frame.payload.ax25_info.body.heater2_manual
    :field heater1_manual: ax25_frame.payload.ax25_info.body.heater1_manual
    :field heater2_on: ax25_frame.payload.ax25_info.body.heater2_on
    :field heater1_on: ax25_frame.payload.ax25_info.body.heater1_on
    :field tab_max: ax25_frame.payload.ax25_info.body.tab_max
    :field tab_min: ax25_frame.payload.ax25_info.body.tab_min
    :field channelon4: ax25_frame.payload.ax25_info.body.channelon4
    :field channelon3: ax25_frame.payload.ax25_info.body.channelon3
    :field channelon2: ax25_frame.payload.ax25_info.body.channelon2
    :field channelon1: ax25_frame.payload.ax25_info.body.channelon1
    :field ich_limit4: ax25_frame.payload.ax25_info.body.ich_limit4
    :field ich_limit3: ax25_frame.payload.ax25_info.body.ich_limit3
    :field ich_limit2: ax25_frame.payload.ax25_info.body.ich_limit2
    :field ich_limit1: ax25_frame.payload.ax25_info.body.ich_limit1
    :field reserved0: ax25_frame.payload.ax25_info.body.reserved0
    :field charger: ax25_frame.payload.ax25_info.body.charger
    :field reserved1: ax25_frame.payload.ax25_info.body.reserved1
    :field uab: ax25_frame.payload.ax25_info.body.uab
    :field reg_tel_id: ax25_frame.payload.ax25_info.body.reg_tel_id
    :field pss_time: ax25_frame.payload.ax25_info.body.pss_time
    :field pss_nres: ax25_frame.payload.ax25_info.body.pss_nres
    :field psdd_fl: ax25_frame.payload.ax25_info.body.psdd_fl
    :field t_amp: ax25_frame.payload.ax25_info.body.t_amp
    :field t_uhf: ax25_frame.payload.ax25_info.body.t_uhf
    :field rssirx: ax25_frame.payload.ax25_info.body.rssirx
    :field rssiidle: ax25_frame.payload.ax25_info.body.rssiidle
    :field pf: ax25_frame.payload.ax25_info.body.pf
    :field pb: ax25_frame.payload.ax25_info.body.pb
    :field uhf_nres: ax25_frame.payload.ax25_info.body.uhf_nres
    :field uhf_fl: ax25_frame.payload.ax25_info.body.uhf_fl
    :field uhf_time: ax25_frame.payload.ax25_info.body.uhf_time
    :field uptime: ax25_frame.payload.ax25_info.body.uptime
    :field current: ax25_frame.payload.ax25_info.body.current
    :field uuhf: ax25_frame.payload.ax25_info.body.uuhf
    :field t_mb: ax25_frame.payload.ax25_info.body.t_mb
    :field mx: ax25_frame.payload.ax25_info.body.mx
    :field my: ax25_frame.payload.ax25_info.body.my
    :field mz: ax25_frame.payload.ax25_info.body.mz
    :field vx: ax25_frame.payload.ax25_info.body.vx
    :field vy: ax25_frame.payload.ax25_info.body.vy
    :field vz: ax25_frame.payload.ax25_info.body.vz
    :field nres: ax25_frame.payload.ax25_info.body.nres
    :field rcon: ax25_frame.payload.ax25_info.body.rcon
    :field fl: ax25_frame.payload.ax25_info.body.fl
    :field time: ax25_frame.payload.ax25_info.body.time
    :field payload_flags: ax25_frame.payload.ax25_info.body.payload_flags
    :field timesend: ax25_frame.payload.ax25_info.body.timesend
    :field t_plate: ax25_frame.payload.ax25_info.body.t_plate
    :field t_cpu: ax25_frame.payload.ax25_info.body.t_cpu
    :field cursens1: ax25_frame.payload.ax25_info.body.cursens1
    :field cursens2: ax25_frame.payload.ax25_info.body.cursens2
    :field nrst: ax25_frame.payload.ax25_info.body.nrst
    :field timerst: ax25_frame.payload.ax25_info.body.timerst
    :field ch1rate: ax25_frame.payload.ax25_info.body.ch1rate
    :field ch2rate: ax25_frame.payload.ax25_info.body.ch2rate
    :field ch3rate: ax25_frame.payload.ax25_info.body.ch3rate
    :field ch4rate: ax25_frame.payload.ax25_info.body.ch4rate
    :field ch5rate: ax25_frame.payload.ax25_info.body.ch5rate
    :field ch6rate: ax25_frame.payload.ax25_info.body.ch6rate
    :field ptrend1: ax25_frame.payload.ax25_info.body.ptrend1
    :field ptrcrt1: ax25_frame.payload.ax25_info.body.ptrcrt1
    :field ptrend2: ax25_frame.payload.ax25_info.body.ptrend2
    :field ptrcrt2: ax25_frame.payload.ax25_info.body.ptrcrt2
    :field ptrend3: ax25_frame.payload.ax25_info.body.ptrend3
    :field ptrcrt3: ax25_frame.payload.ax25_info.body.ptrcrt3
    :field lastevent_ch1_1: ax25_frame.payload.ax25_info.body.lastevent_ch1_1
    :field lastevent_ch1_2: ax25_frame.payload.ax25_info.body.lastevent_ch1_2
    :field lastevent_ch1_3: ax25_frame.payload.ax25_info.body.lastevent_ch1_3
    :field lastevent_ch2_1: ax25_frame.payload.ax25_info.body.lastevent_ch2_1
    :field lastevent_ch2_2: ax25_frame.payload.ax25_info.body.lastevent_ch2_2
    :field lastevent_ch2_3: ax25_frame.payload.ax25_info.body.lastevent_ch2_3
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Siriussat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Siriussat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Siriussat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Siriussat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Siriussat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Siriussat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Siriussat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Siriussat.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Siriussat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Siriussat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Siriussat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Siriussat.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Siriussat.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Siriussat.Tlm(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Siriussat.Tlm(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Beacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.usb1 = self._io.read_u2le()
            self.usb2 = self._io.read_u2le()
            self.usb3 = self._io.read_u2le()
            self.isb1 = self._io.read_u2le()
            self.isb2 = self._io.read_u2le()
            self.isb3 = self._io.read_u2le()
            self.iab = self._io.read_s2le()
            self.ich1 = self._io.read_u2le()
            self.ich2 = self._io.read_u2le()
            self.ich3 = self._io.read_u2le()
            self.ich4 = self._io.read_u2le()
            self.t1_pw = self._io.read_s2le()
            self.t2_pw = self._io.read_s2le()
            self.t3_pw = self._io.read_s2le()
            self.t4_pw = self._io.read_s2le()
            self.uab_crit = self._io.read_bits_int_be(1) != 0
            self.uab_min = self._io.read_bits_int_be(1) != 0
            self.heater2_manual = self._io.read_bits_int_be(1) != 0
            self.heater1_manual = self._io.read_bits_int_be(1) != 0
            self.heater2_on = self._io.read_bits_int_be(1) != 0
            self.heater1_on = self._io.read_bits_int_be(1) != 0
            self.tab_max = self._io.read_bits_int_be(1) != 0
            self.tab_min = self._io.read_bits_int_be(1) != 0
            self.channelon4 = self._io.read_bits_int_be(1) != 0
            self.channelon3 = self._io.read_bits_int_be(1) != 0
            self.channelon2 = self._io.read_bits_int_be(1) != 0
            self.channelon1 = self._io.read_bits_int_be(1) != 0
            self.ich_limit4 = self._io.read_bits_int_be(1) != 0
            self.ich_limit3 = self._io.read_bits_int_be(1) != 0
            self.ich_limit2 = self._io.read_bits_int_be(1) != 0
            self.ich_limit1 = self._io.read_bits_int_be(1) != 0
            self.reserved0 = self._io.read_bits_int_be(7)
            self.charger = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.reserved1 = self._io.read_u1()
            self.uab = self._io.read_s2le()
            self.reg_tel_id = self._io.read_u4le()
            self.pss_time = self._io.read_s4le()
            self.pss_nres = self._io.read_u1()
            self.psdd_fl = self._io.read_u1()
            self.t_amp = self._io.read_s1()
            self.t_uhf = self._io.read_s1()
            self.rssirx = self._io.read_s1()
            self.rssiidle = self._io.read_s1()
            self.pf = self._io.read_s1()
            self.pb = self._io.read_s1()
            self.uhf_nres = self._io.read_u1()
            self.uhf_fl = self._io.read_u1()
            self.uhf_time = self._io.read_s4le()
            self.uptime = self._io.read_u4le()
            self.current = self._io.read_u2le()
            self.uuhf = self._io.read_s2le()


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Siriussat.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Siriussat.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Siriussat.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class Extendedbeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.t_mb = self._io.read_s2le()
            self.mx = self._io.read_f4le()
            self.my = self._io.read_f4le()
            self.mz = self._io.read_f4le()
            self.vx = self._io.read_f4le()
            self.vy = self._io.read_f4le()
            self.vz = self._io.read_f4le()
            self.nres = self._io.read_u1()
            self.rcon = self._io.read_u1()
            self.fl = self._io.read_u1()
            self.time = self._io.read_s4le()
            self.payload_flags = self._io.read_u1()
            self.timesend = self._io.read_s4le()
            self.t_plate = self._io.read_s2le()
            self.t_cpu = self._io.read_s2le()
            self.cursens1 = self._io.read_u2le()
            self.cursens2 = self._io.read_u2le()
            self.nrst = self._io.read_u1()
            self.timerst = self._io.read_s4le()
            self.ch1rate = self._io.read_u2le()
            self.ch2rate = self._io.read_u2le()
            self.ch3rate = self._io.read_u2le()
            self.ch4rate = self._io.read_u2le()
            self.ch5rate = self._io.read_u2le()
            self.ch6rate = self._io.read_u2le()
            self.ptrend1 = self._io.read_u2le()
            self.ptrcrt1 = self._io.read_u2le()
            self.ptrend2 = self._io.read_u2le()
            self.ptrcrt2 = self._io.read_u2le()
            self.ptrend3 = self._io.read_u2le()
            self.ptrcrt3 = self._io.read_u2le()
            self.lastevent_ch1_1 = self._io.read_u1()
            self.lastevent_ch1_2 = self._io.read_u1()
            self.lastevent_ch1_3 = self._io.read_u1()
            self.lastevent_ch2_1 = self._io.read_u1()
            self.lastevent_ch2_2 = self._io.read_u1()
            self.lastevent_ch2_3 = self._io.read_u1()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Siriussat.Callsign(_io__raw_callsign_ror, self, self._root)


    class Tlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_type = self._io.read_u1()
            self.unknown1 = self._io.read_bytes(7)
            _on = self.beacon_type
            if _on == 22:
                self.body = Siriussat.Beacon(self._io, self, self._root)
            elif _on == 23:
                self.body = Siriussat.Extendedbeacon(self._io, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Kosen1(KaitaiStruct):
    """:field callsign: callsign
    :field s: s
    :field rssi: rssi
    :field battery_temperature: bat_t
    :field battery_voltage: bat_v
    :field battery_current: bat_i
    :field load_current: load_i
    :field solar_cell_output_current_minus_z: sc_z
    :field solar_cell_output_current_minus_y: sc_y
    :field solar_cell_output_current_minus_x: sc_x
    :field necessary_for_lengthcheck: necessary_for_lengthcheck
    :field beacon: beacon
    
    .. seealso::
       Source - https://space.kochi-ct.jp/kosen-1/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.callsign = (self._io.read_bytes(8)).decode(u"ASCII")
        if not self.callsign == u"kosen-1 ":
            raise kaitaistruct.ValidationNotEqualError(u"kosen-1 ", self.callsign, self._io, u"/seq/0")
        self.s = (self._io.read_bytes(2)).decode(u"ASCII")
        if not  ((self.s == u"s1") or (self.s == u"s2") or (self.s == u"S1") or (self.s == u"S2")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.s, self._io, u"/seq/1")
        self.value1 = self._io.read_u1()
        self.value2 = self._io.read_u1()
        self.value3 = self._io.read_u1()
        self.value4 = self._io.read_u1()
        self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

    @property
    def bat_v(self):
        if hasattr(self, '_m_bat_v'):
            return self._m_bat_v

        if  ((self.value3 != 255) and (self.s == u"s1")) :
            self._m_bat_v = (0.0253 * self.value3)

        return getattr(self, '_m_bat_v', None)

    @property
    def value3_hex_right_digit(self):
        if hasattr(self, '_m_value3_hex_right_digit'):
            return self._m_value3_hex_right_digit

        self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
        return getattr(self, '_m_value3_hex_right_digit', None)

    @property
    def necessary_for_lengthcheck(self):
        if hasattr(self, '_m_necessary_for_lengthcheck'):
            return self._m_necessary_for_lengthcheck

        if len(self.lengthcheck) != 0:
            self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

        return getattr(self, '_m_necessary_for_lengthcheck', None)

    @property
    def value1_hex_left(self):
        if hasattr(self, '_m_value1_hex_left'):
            return self._m_value1_hex_left

        self._m_value1_hex_left = self.value1 // 16
        return getattr(self, '_m_value1_hex_left', None)

    @property
    def value2_hex_right(self):
        if hasattr(self, '_m_value2_hex_right'):
            return self._m_value2_hex_right

        self._m_value2_hex_right = (self.value2 % 16)
        return getattr(self, '_m_value2_hex_right', None)

    @property
    def value3_hex(self):
        if hasattr(self, '_m_value3_hex'):
            return self._m_value3_hex

        self._m_value3_hex = (u".." if self.value3_hex_left_digit + self.value3_hex_right_digit == u"ff" else self.value3_hex_left_digit + self.value3_hex_right_digit)
        return getattr(self, '_m_value3_hex', None)

    @property
    def rssi(self):
        if hasattr(self, '_m_rssi'):
            return self._m_rssi

        if  ((self.value1 != 255) and (self.s == u"s1")) :
            self._m_rssi = ((0.409 * self.value1) - 127.4)

        return getattr(self, '_m_rssi', None)

    @property
    def value3_hex_right(self):
        if hasattr(self, '_m_value3_hex_right'):
            return self._m_value3_hex_right

        self._m_value3_hex_right = (self.value3 % 16)
        return getattr(self, '_m_value3_hex_right', None)

    @property
    def value1_hex_right_digit(self):
        if hasattr(self, '_m_value1_hex_right_digit'):
            return self._m_value1_hex_right_digit

        self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
        return getattr(self, '_m_value1_hex_right_digit', None)

    @property
    def beacon(self):
        if hasattr(self, '_m_beacon'):
            return self._m_beacon

        self._m_beacon = self.s + u" " + self.value1_hex + u" " + self.value2_hex + u" " + self.value3_hex + u" " + self.value4_hex
        return getattr(self, '_m_beacon', None)

    @property
    def sc_x(self):
        if hasattr(self, '_m_sc_x'):
            return self._m_sc_x

        if  ((self.value4 != 255) and (self.s == u"s2")) :
            self._m_sc_x = (self.value4 / 0.256)

        return getattr(self, '_m_sc_x', None)

    @property
    def value3_hex_left(self):
        if hasattr(self, '_m_value3_hex_left'):
            return self._m_value3_hex_left

        self._m_value3_hex_left = self.value3 // 16
        return getattr(self, '_m_value3_hex_left', None)

    @property
    def load_i(self):
        if hasattr(self, '_m_load_i'):
            return self._m_load_i

        if  ((self.value1 != 255) and (self.s == u"s2")) :
            self._m_load_i = (self.value1 / 0.1024)

        return getattr(self, '_m_load_i', None)

    @property
    def bat_i(self):
        if hasattr(self, '_m_bat_i'):
            return self._m_bat_i

        if  ((self.value4 != 255) and (self.s == u"s1")) :
            self._m_bat_i = ((self.value4 - 128) / 0.0512)

        return getattr(self, '_m_bat_i', None)

    @property
    def sc_z(self):
        if hasattr(self, '_m_sc_z'):
            return self._m_sc_z

        if  ((self.value2 != 255) and (self.s == u"s2")) :
            self._m_sc_z = (self.value2 / 0.256)

        return getattr(self, '_m_sc_z', None)

    @property
    def value4_hex_left_digit(self):
        if hasattr(self, '_m_value4_hex_left_digit'):
            return self._m_value4_hex_left_digit

        self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
        return getattr(self, '_m_value4_hex_left_digit', None)

    @property
    def value1_hex_right(self):
        if hasattr(self, '_m_value1_hex_right'):
            return self._m_value1_hex_right

        self._m_value1_hex_right = (self.value1 % 16)
        return getattr(self, '_m_value1_hex_right', None)

    @property
    def value4_hex_right(self):
        if hasattr(self, '_m_value4_hex_right'):
            return self._m_value4_hex_right

        self._m_value4_hex_right = (self.value4 % 16)
        return getattr(self, '_m_value4_hex_right', None)

    @property
    def value4_hex_right_digit(self):
        if hasattr(self, '_m_value4_hex_right_digit'):
            return self._m_value4_hex_right_digit

        self._m_value4_hex_right_digit = (u"a" if str(self.value4_hex_right) == u"10" else (u"b" if str(self.value4_hex_right) == u"11" else (u"c" if str(self.value4_hex_right) == u"12" else (u"d" if str(self.value4_hex_right) == u"13" else (u"e" if str(self.value4_hex_right) == u"14" else (u"f" if str(self.value4_hex_right) == u"15" else str(self.value4_hex_right)))))))
        return getattr(self, '_m_value4_hex_right_digit', None)

    @property
    def value1_hex(self):
        if hasattr(self, '_m_value1_hex'):
            return self._m_value1_hex

        self._m_value1_hex = (u".." if self.value1_hex_left_digit + self.value1_hex_right_digit == u"ff" else self.value1_hex_left_digit + self.value1_hex_right_digit)
        return getattr(self, '_m_value1_hex', None)

    @property
    def value2_hex_right_digit(self):
        if hasattr(self, '_m_value2_hex_right_digit'):
            return self._m_value2_hex_right_digit

        self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
        return getattr(self, '_m_value2_hex_right_digit', None)

    @property
    def value4_hex_left(self):
        if hasattr(self, '_m_value4_hex_left'):
            return self._m_value4_hex_left

        self._m_value4_hex_left = self.value4 // 16
        return getattr(self, '_m_value4_hex_left', None)

    @property
    def value2_hex_left_digit(self):
        if hasattr(self, '_m_value2_hex_left_digit'):
            return self._m_value2_hex_left_digit

        self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
        return getattr(self, '_m_value2_hex_left_digit', None)

    @property
    def value2_hex(self):
        if hasattr(self, '_m_value2_hex'):
            return self._m_value2_hex

        self._m_value2_hex = (u".." if self.value2_hex_left_digit + self.value2_hex_right_digit == u"ff" else self.value2_hex_left_digit + self.value2_hex_right_digit)
        return getattr(self, '_m_value2_hex', None)

    @property
    def value4_hex(self):
        if hasattr(self, '_m_value4_hex'):
            return self._m_value4_hex

        self._m_value4_hex = (u".." if self.value4_hex_left_digit + self.value4_hex_right_digit == u"ff" else self.value4_hex_left_digit + self.value4_hex_right_digit)
        return getattr(self, '_m_value4_hex', None)

    @property
    def bat_t(self):
        if hasattr(self, '_m_bat_t'):
            return self._m_bat_t

        if  ((self.value2 != 255) and (self.s == u"s1")) :
            self._m_bat_t = ((((((0.00003 * self.value2) * self.value2) * self.value2) - ((0.011 * self.value2) * self.value2)) + (1.49 * self.value2)) - 28.1)

        return getattr(self, '_m_bat_t', None)

    @property
    def value3_hex_left_digit(self):
        if hasattr(self, '_m_value3_hex_left_digit'):
            return self._m_value3_hex_left_digit

        self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
        return getattr(self, '_m_value3_hex_left_digit', None)

    @property
    def value2_hex_left(self):
        if hasattr(self, '_m_value2_hex_left'):
            return self._m_value2_hex_left

        self._m_value2_hex_left = self.value2 // 16
        return getattr(self, '_m_value2_hex_left', None)

    @property
    def value1_hex_left_digit(self):
        if hasattr(self, '_m_value1_hex_left_digit'):
            return self._m_value1_hex_left_digit

        self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
        return getattr(self, '_m_value1_hex_left_digit', None)

    @property
    def sc_y(self):
        if hasattr(self, '_m_sc_y'):
            return self._m_sc_y

        if  ((self.value3 != 255) and (self.s == u"s2")) :
            self._m_sc_y = (self.value3 / 0.256)

        return getattr(self, '_m_sc_y', None)


